<?php

namespace app\models;

use Yii;

/**
 * This is the model class for table "Vehiculo".
 *
 * @property string $id
 * @property string|null $nombre
 * @property int|null $ano
 * @property string|null $idColor
 * @property string|null $placa
 * @property string|null $serie
 * @property string|null $creado
 * @property string|null $modificado
 * @property string|null $eliminado
 * @property string|null $idTipo
 * @property string|null $idMarca
 * @property string|null $descripcion
 *
 * @property Marca $idMarca0
 * @property TipoAuto $idTipo0
 * @property MediaVehiculo[] $mediaVehiculos
 * @property Servicio[] $servicios
 */
class Vehiculo extends ModeloBase
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'Vehiculo';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id'], 'required'],
            [['ano'], 'default', 'value' => null],
            [['ano'], 'integer'],
            [['creado', 'modificado', 'eliminado'], 'safe'],
            [['id','idMarca','idColor','idTipo'], 'string', 'max' => 36],
            [['nombre',  'placa', 'serie','descripcion'], 'string', 'max' => 100],
            [['id'], 'unique'],
            [['idMarca'], 'exist', 'skipOnError' => true, 'targetClass' => Marca::class, 'targetAttribute' => ['idMarca' => 'id']],
            [['idColor'], 'exist', 'skipOnError' => true, 'targetClass' => Color::class, 'targetAttribute' => ['idColor' => 'id']],
            [['idTipo'], 'exist', 'skipOnError' => true, 'targetClass' => TipoAuto::class, 'targetAttribute' => ['idTipo' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'nombre' => 'Nombre',
            'ano' => 'Ano',
            'idColor' => 'ID Color',
            'placa' => 'Placa',
            'serie' => 'Serie',
            'creado' => 'Creado',
            'idTipo' => 'ID Tipo',
            'idMarca' => 'ID Marca',
            'descripcion' => 'Descripcion',
            'modificado' => 'Modificado',
            'eliminado' => 'Eliminado',
        ];
    }

    public function getMarca()
    {
        return $this->hasOne(Marca::class, ['id' => 'idMarca']);
    }

    public function getTipo()
    {
        return $this->hasOne(TipoAuto::class, ['id' => 'idTipo']);
    }

    public function getMediaVehiculo()
    {
        return $this->hasMany(MediaVehiculo::class, ['idVehiculo' => 'id']);
    }

    public function getServicio()
    {
        return $this->hasMany(Servicio::class, ['idVehiculo' => 'id']);
    }

    public function getMedia() {
        return $this->hasMany(Media::class, ['idMedia' => 'id'])
        ->viaTable('MediaVehiculo', ['idVehiculo' => 'idVehiculo']);
    }
}
