<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class HerramientaController extends AuthController {

  public $modelClass = "v1\models\Herramienta";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $tipoherramienta = trim($this->req->get("tipo", ""));

    $query = $this->queryInicial;

    if($id > 0) {
      $query->andWhere(["id" => $id]);
    }

    if($tipoherramienta) {
        $query->andWhere(["[[TipoHerramienta]].{{id}}", $tipoherramienta]);
    }

    if($buscar) {
      
      $query->andWhere([
        "OR",
        ["ilike", "nombre", $buscar]
      ]);

    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }
    if($modelo === null) {
      $modelo = new $this->modelClass();
      $modelo->uuid();
      $modelo->creado = new Expression('now()');
    } else {
      $modelo->modificado = new Expression('now()');
    }

    $modelo->load($this->req->getBodyParams(), '');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("Hubo un problema al guardar la herramienta");
    }

    $modelo->refresh();
    return (new Respuesta($modelo))
      ->mensaje("Herramienta guardada");
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Herramienta no encontrada");
    }
    $modelo->eliminado = new Expression('now()');
    if(!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar herramienta");
    }

    return (new Respuesta())
      ->mensaje("Herramienta eliminada");
  }
}