<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class TipoServicioController extends AuthController
{

  public $modelClass = "v1\models\TipoServicio";

  public function actionIndex()
  {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));

    $query = $this->queryInicial;


    if ($id !== "") {
      $query->andWhere(["id" => $id]);
    }

    if ($buscar) {

      $query->andWhere([
        "OR",
        ["ilike", "nombre", $buscar]
      ]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar()
  {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }
    if ($modelo === null) {
      $modelo = new $this->modelClass();
      $modelo->uuid();
      $modelo->creado = new Expression('now()');
    } else {
      $modelo->modificado = new Expression('now()');
    }
    $modelo->load($this->req->getBodyParams(), '');

    $clave = $modelo->clave;  
    $existeClave = $this->modelClass::find()->where(['clave' => $clave])->andWhere(['<>', 'id', $id])->exists();

    if ($existeClave) {
      return (new Respuesta())
        ->esError()
        ->mensaje("La clave '$clave' ya existe. Por favor, ingresa una clave diferente.");
    }
    $modelo->load($this->req->getBodyParams(), '');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("Hubo un problema al guardar el vehiculo");
    }

    $modelo->refresh();
    return (new Respuesta($modelo))
      ->mensaje("Vehiculo guardado");
  }

  public function actionEliminar()
  {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Tipo de Servicio no encontrado");
    }
    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar Tipo de Servicio");
    }

    return (new Respuesta())
      ->mensaje("Tipo de Servicio eliminada");
  }
}
