<?php

namespace v1\controllers;

use app\models\MediaVehiculo;
use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class VehiculoController extends AuthController {

  public $modelClass = "v1\models\Vehiculo";
  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $tipo = trim($this->req->get("tipo", ""));
    $marca = trim($this->req->get("marca", ""));

    $query = $this->queryInicial;

    if ($id !== "") {
        $query->andWhere(["id" => $id]);
    }

    if ($buscar) {
        // Ejemplo de buscador
        $query->andWhere([
            "OR",
            ["ilike", "nombre", $buscar]
        ]);
        //
    }

    if ($tipo !== "") {
        $query->andWhere(["tipo" => $tipo]);
    }

    if ($marca !== "") {
        $query->andWhere(["marca" => $marca]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
}

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $imagenes = $this->req->getBodyParam("imagenes", []);
		$archivos = $this->req->getBodyParam("archivos", []);
    $modelo = null;

    $transaccion = \Yii::$app->db->beginTransaction();
		try {
      if($id !== "") {
        $modelo = $this->modelClass::findOne($id);
      }
      if($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

			$modelo->load($this->req->getBodyParams(), '');
			if (!$modelo->save()) {
				$transaccion->rollBack();
				return (new Respuesta($modelo))
					->esError()
					->mensaje("Hubo un problema al guardar el Vehiculo");
			}

			MediaVehiculo::deleteAll(['id' => $modelo->id]);

			foreach ($imagenes as $item) {
				$id = $item['idMedia'];
				$interMedia = new MediaVehiculo();

				$interMedia->idMedia = $id;
				$interMedia->idVehiculo = $modelo->id;

				$interMedia->tipo = "IMAGEN";

				if (!$interMedia->save()) {
					$transaccion->rollBack();
					return (new Respuesta($interMedia))
						->mensaje("Hubo un problema al guardar la imagen {$item['nombre']}");
				}
			}

			foreach ($archivos as $item) {
				$id = $item['idMedia'];
				$interMedia = new MediaVehiculo();

				$interMedia->idMedia = $id;
				$interMedia->idVehiculo = $modelo->id;
				$interMedia->tipo = "ARCHIVO";

				if (!$interMedia->save()) {
					$transaccion->rollBack();
					return (new Respuesta($interMedia))
						->mensaje("Hubo un problema al guardar el archivo {$item['nombre']}");
				}
			}

			$transaccion->commit();
			$modelo->refresh();
			return (new Respuesta($modelo))
				->mensaje("Vehiculo ha sido guardado de manera exitosa");
		} catch (\Exception $e) {
			$transaccion->rollBack();
			return (new Respuesta($modelo))
				->esError()
				->mensaje($e->getMessage());
		}
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Vehiculo no encontrado");
    }
    $modelo->eliminado = new Expression('now()');
    if(!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar vehiculo");
    }

    return (new Respuesta())
      ->mensaje("Vehiculo eliminado");
  }
}